searchData={"content_type":"text/markdown","items":[{"doc":"An ExBuffer is a process that maintains a collection of items and flushes\nthem once certain conditions have been met.\n\nExBuffers can flush based on a timeout, a maximum length (item count), a\nmaximum byte size, or a combination of the three. When multiple conditions are\nused, the ExBuffer will flush when the **first** condition is met.\n\nExBuffers also come with a number of helpful tools for testing and debugging.","ref":"ExBuffer.html","title":"ExBuffer","type":"module"},{"doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"ExBuffer.html#child_spec/1","title":"ExBuffer.child_spec/1","type":"function"},{"doc":"Lazily chunks an enumerable based on one or more ExBuffer flush conditions.\n\nThis function currently supports length and size conditions. If multiple\nconditions are specified, a chunk is emitted once the **first** condition is\nmet (just like an ExBuffer process).\n\nWhile this function is useful in it's own right, it's included primarily as\nanother way to synchronously test applications that use ExBuffer.","ref":"ExBuffer.html#chunk_enum!/2","title":"ExBuffer.chunk_enum!/2","type":"function"},{"doc":"An enumerable can be chunked with the following options:\n\n  * `:max_length` - The maximum allowed length (item count) of a chunk.\n    (Optional `non_neg_integer()`, Default = `:infinity`)\n\n  * `:max_size` - The maximum allowed size (in bytes) of a chunk. (Optional\n    `non_neg_integer()`, Default = `:infinity`)\n\n> ##","ref":"ExBuffer.html#chunk_enum!/2-options","title":"Options - ExBuffer.chunk_enum!/2","type":"function"},{"doc":">\n> Not including either of the options above is permitted but will result in a\n> single chunk being emitted. One can achieve a similar result in a more performant\n> way using `Stream.into/2`. In that same vein, including only a `:max_length`\n> condition makes this function a less performant version of `Stream.chunk_every/2`.\n> This function is optimized for chunking by either size or size **and** count. Any other\n> chunking strategy can likely be achieved in a more efficient way using other methods.","ref":"ExBuffer.html#chunk_enum!/2-warning-warning","title":"Warning {: .warning} - ExBuffer.chunk_enum!/2","type":"function"},{"doc":"enum = [\"foo\", \"bar\", \"baz\", \"foobar\", \"barbaz\", \"foobarbaz\"]\n\n    enum\n    |> ExBuffer.chunk_enum!(max_length: 3, max_size: 10)\n    |> Enum.into([])\n    #=> [[\"foo\", \"bar\", \"baz\"], [\"foobar\", \"barbaz\"], [\"foobarbaz\"]]","ref":"ExBuffer.html#chunk_enum!/2-example","title":"Example - ExBuffer.chunk_enum!/2","type":"function"},{"doc":"Dumps the contents of the given `ExBuffer` to a list, bypassing a flush\ncallback and resetting the buffer.\n\nWhile this behavior may occasionally be desriable in a production environment,\nit is intended to be used primarily for testing and debugging.","ref":"ExBuffer.html#dump/1","title":"ExBuffer.dump/1","type":"function"},{"doc":"ExBuffer.insert(:test_buffer, \"foo\")\n    ExBuffer.insert(:test_buffer, \"bar\")\n\n    ExBuffer.dump(:test_buffer)\n    #=> [\"foo\", \"bar\"]\n\n    ExBuffer.length(:test_buffer)\n    #=> 0","ref":"ExBuffer.html#dump/1-example","title":"Example - ExBuffer.dump/1","type":"function"},{"doc":"Flushes the given `ExBuffer`, regardless of whether or not the flush conditions\nhave been met.\n\nWhile this behavior may occasionally be desriable in a production environment,\nit is intended to be used primarily for testing and debugging.","ref":"ExBuffer.html#flush/2","title":"ExBuffer.flush/2","type":"function"},{"doc":"An ExBuffer can be flushed with the following options:\n\n  * `:async` - Determines whether or not the flush will be asynchronous. (Optional\n    `boolean()`, Default = `true`)","ref":"ExBuffer.html#flush/2-options","title":"Options - ExBuffer.flush/2","type":"function"},{"doc":"ExBuffer.insert(:test_buffer, \"foo\")\n    ExBuffer.insert(:test_buffer, \"bar\")\n\n    # Assuming the flush callback is `IO.inspect/1`\n    ExBuffer.flush(:test_buffer)\n    #=> outputs [\"foo\", \"bar\"]\n\n    ExBuffer.length(:test_buffer)\n    #=> 0","ref":"ExBuffer.html#flush/2-example","title":"Example - ExBuffer.flush/2","type":"function"},{"doc":"Inserts the given item into the given `ExBuffer`.","ref":"ExBuffer.html#insert/2","title":"ExBuffer.insert/2","type":"function"},{"doc":"ExBuffer.insert(:test_buffer, \"foo\")\n    #=> :test_buffer items = [\"foo\"]\n\n    ExBuffer.insert(:test_buffer, \"bar\")\n    #=> :test_buffer items = [\"foo\", \"bar\"]","ref":"ExBuffer.html#insert/2-example","title":"Example - ExBuffer.insert/2","type":"function"},{"doc":"Returns the length (item count) of the given `ExBuffer`.\n\nWhile this behavior may occasionally be desriable in a production environment,\nit is intended to be used primarily for testing and debugging.","ref":"ExBuffer.html#length/1","title":"ExBuffer.length/1","type":"function"},{"doc":"ExBuffer.insert(:test_buffer, \"foo\")\n    ExBuffer.insert(:test_buffer, \"bar\")\n\n    ExBuffer.length(:test_buffer)\n    #=> 2","ref":"ExBuffer.html#length/1-example","title":"Example - ExBuffer.length/1","type":"function"},{"doc":"Retuns the size (in bytes) of the given `ExBuffer`.\n\nItem size is computed using `Kernel.byte_size/1`. Because this function requires\na bitstring input, non-bitstring items are first transformed into binary using\n`:erlang.term_to_binary/1`.\n\nWhile this behavior may occasionally be desriable in a production environment,\nit is intended to be used primarily for testing and debugging.","ref":"ExBuffer.html#size/1","title":"ExBuffer.size/1","type":"function"},{"doc":"ExBuffer.insert(:test_buffer, \"foo\")\n    ExBuffer.insert(:test_buffer, \"bar\")\n\n    ExBuffer.size(:test_buffer)\n    #=> 6","ref":"ExBuffer.html#size/1-example","title":"Example - ExBuffer.size/1","type":"function"},{"doc":"Starts an `ExBuffer` process linked to the current process.","ref":"ExBuffer.html#start_link/1","title":"ExBuffer.start_link/1","type":"function"},{"doc":"An ExBuffer can be started with the following options:\n\n  * `:callback` - The function that will be invoked to handle a flush. This\n    function should expect a single parameter: a list of items. (Required\n    `function()`)\n\n  * `:buffer_timeout` - The maximum time (in ms) allowed between flushes of\n    the ExBuffer. Once this amount of time has passed, the ExBuffer will be\n    flushed. (Optional `non_neg_integer()`, Default = `:infinity`)\n\n  * `:max_length` - The maximum allowed length (item count) of the ExBuffer.\n    Once the limit is hit, the ExBuffer will be flushed. (Optional\n    `non_neg_integer()`, Default = `:infinity`)\n\n  * `:max_size` - The maximum allowed size (in bytes) of the ExBuffer. Once\n    the limit is hit (or exceeded), the ExBuffer will be flushed. For more\n    information on how size is computed, see `ExBuffer.size/1`. (Optional\n    `non_neg_integer()`, Default = `:infinity`)\n\nAdditionally, an ExBuffer can also be started with any `GenServer` options.","ref":"ExBuffer.html#start_link/1-options","title":"Options - ExBuffer.start_link/1","type":"function"},{"doc":"","ref":"ExBuffer.html#t:t/0","title":"ExBuffer.t/0","type":"type"},{"doc":"# ExBuffer\n\nAn ExBuffer is a process that maintains a collection of items and flushes them once certain conditions have been met.","ref":"readme.html","title":"ExBuffer","type":"extras"},{"doc":"This package can be installed by adding `:ex_buffer` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:ex_buffer \"~> 0.1.0\"}\n  ]\nend\n```","ref":"readme.html#installation","title":"Installation - ExBuffer","type":"extras"},{"doc":"For additional documentation, see [HexDocs](https://hexdocs.pm/ex_buffer/readme.html).","ref":"readme.html#documentation","title":"Documentation - ExBuffer","type":"extras"},{"doc":"We can easily start an ExBuffer by adding it directly to a supervision tree.\n\n```elixir\nopts = [callback: &IO.inspect/1, buffer_timeout: 5_000, name: :test_buffer]\n\nchildren = [\n  {ExBuffer, opts}\n]\n\nSupervisor.start_link(children, strategy: :one_for_one)\n```\n\nOnce the buffer has been started, we can insert items.\n\n```elixir\nExBuffer.insert(:test_buffer, \"foo\")\nExBuffer.insert(:test_buffer, \"bar\")\n```\n\nAnd, once any of the configured conditions have been met, the buffer will automatically flush.\n\n```elixir\n# 5 seconds after previous flush\n#=> outputs [\"foo\", \"bar\"]\n\nExBuffer.insert(:test_buffer, \"baz\")\n\n# 5 seconds after previous flush\n#=> outputs [\"baz\"]\n```","ref":"readme.html#getting-started","title":"Getting Started - ExBuffer","type":"extras"}]}